/*
 * thrulay.h -- network throughput tester (the client part), headers.
 *
 * Written by Bernhard Lutzmann, belu@users.sf.net
 *
 * @(#) $Id: thrulay.h,v 1.1.1.1 2005/10/02 01:18:09 hliu Exp $
 *
 * Copyright 2005, Internet2.
 * Legal conditions are in file LICENSE
 * (MD5 = ecfa50d1b0bfbb81b658c810d0476a52).
 */

#ifndef THRULAY_THRULAY_H_INCLUDED
#define THRULAY_THRULAY_H_INCLUDED

#define	STREAMS_MAX	256

/* Client options */
struct _opt {
	char *server_name;	/* Where to send test traffic. */
	int num_streams;	/* Number of test streams to start. */
	int test_duration;	/* For how long, in seconds. */
	int reporting_interval;	/* How often to report progress. */
	int window;		/* Window size, in bytes. */
	int block_size;		/* Block size, in bytes. */
	int port;		/* Server TCP port number. */
	uint64_t rate;		/* Rate in b/s.  Non-zero means UDP. */
	uint8_t dscp;		/* DSCP: 6 bit field */
	int busywait;		/* Busy wait in UDP test. */
} opt;

/* Stream information */
struct _stream {
	/* Connection socket */
	int sock;
} stream[STREAMS_MAX];

/* Statistics information (per stream) */
struct _stat {
	/* Block counter */
	unsigned int blocks_since_first;
	unsigned int blocks_since_last;

	/* RTT */
	double min_rtt_since_first;
	double min_rtt_since_last;
	double max_rtt_since_first;
	double max_rtt_since_last;
	double tot_rtt_since_first;
	double tot_rtt_since_last;
} stats[STREAMS_MAX];

/* Timer information */
struct _timer {
	struct timeval start;
	struct timeval stop;
	struct timeval next;
	struct timeval last;
	double runtime;
} timer;

/*
 * Prototypes
 */
/* API BEGIN */
/* (API) General functions */
void
thrulay_init(void);

void
thrulay_exit(void);

void
thrulay_start(void);

void
thrulay_report_final(void);

/* (API) TCP tests */
void
thrulay_tcp_init(void);

void
thrulay_tcp_init_id(int);

void
thrulay_tcp_exit(void);

void
thrulay_tcp_exit_id(int);

void
thrulay_tcp_start(void);

void
thrulay_tcp_stop(void);

void
thrulay_tcp_report(void);

void
thrulay_tcp_report_id(int);

void
thrulay_tcp_report_final(void);

void
thrulay_tcp_report_final_id(int);

/* (API) UDP test */
void
thrulay_udp_init(void);

void
thrulay_udp_exit(void);

void
thrulay_udp_start(void);

void
thrulay_udp_report_final(void);

/* API END */

/* Timer */
void
timer_start(void);

void
timer_stop(void);

void
timer_check(void);

void
timer_report(struct timeval *);

void
timer_end(struct timeval *);


/* Options and statistics */
void
options_init(void);

void
tcp_stats_init(void);

#endif /* ifndef THRULAY_THRULAY_H_INCLUDED */
