/*
 * util.h -- common utility routines for thrulay, headers.
 * 
 * Written by Stanislav Shalunov, http://www.internet2.edu/~shalunov/
 *            Bernhard Lutzmann, belu@users.sf.net
 * 
 * @(#) $Id: util.h,v 1.1.1.1 2005/10/02 01:18:09 hliu Exp $
 * 
 * Copyright 2003, Internet2.
 * Legal conditions are in file LICENSE
 * (MD5 = ecfa50d1b0bfbb81b658c810d0476a52).
 */

#ifndef THRULAY_UTIL_H_INCLUDED
#define THRULAY_UTIL_H_INCLUDED

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef HAVE_STDINT_H
#include <stdint.h>
#endif

#ifdef HAVE_FASTTIME_H
#include <fasttime.h>
#endif

#ifdef HAVE_TSCI2_H
#include <tsci2.h>
#endif

#include <sys/types.h>
#include <sys/uio.h>
#include <sys/time.h>
#include <unistd.h>

#define BLOCK_HEADER	16
#define MIN_BLOCK	BLOCK_HEADER
#define MAX_BLOCK	1048576

#define ERR_FATAL	0
#define ERR_WARNING	1

void
error(int, const char *);

ssize_t
read_exactly(int, void *, size_t);

ssize_t
write_exactly(int, const void *, size_t);

double
time_diff(const struct timeval *, const struct timeval *);

int
set_window_size_directed(int, int, int);

int
set_window_size(int, int);

int
set_dscp(int, uint8_t);

void
tv2ntp(const struct timeval *tv, char *);

void
ntp2tv(struct timeval *tv, const char *);

int
normalize_tv(struct timeval *);

/*
 * We define `tsc_init()' and `tsc_gettimeofday()' here. If a TSC library is
 * installed, we use it. If none is found, we use good old `gettimeofday()'.
 *
 * Supported TSC libraries:
 *   fasttime: http://fasttime.sf.net/
 *   TSC-I2: http://tsc-xluo.sf.net/
 */
#if defined(HAVE_FASTTIME_H) && defined(HAVE_LIBFASTTIME)
#define tsc_init()		fasttime_init()
#define tsc_gettimeofday(tv)	fasttime_gettimeofday(tv)
#elif defined(HAVE_TSCI2_H) && defined(HAVE_LIBTSCI2)
#define tsc_init()		tsci2_init(TSCI2_DAEMON | \
				           TSCI2_CLIENT | \
				           TSCI2_FALLBACK)
#define tsc_gettimeofday(tv)	tsci2_gettimeofday(tv, 0)
#else
#define tsc_init()		;
#define tsc_gettimeofday(tv)	gettimeofday(tv, 0)
#endif

#endif /* ifndef THRULAY_UTIL_H_INCLUDED */
